import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:payment_helper/src/ClientPaymentService.dart';
import 'package:payment_helper/src/ServerPaymentService.dart';
import 'package:stripe_payment/stripe_payment.dart';

abstract class PaymentService {

  static Future<void> _savePayment(double amount, String currency) async {
    final String uid = (await FirebaseAuth.instance.currentUser()).uid;

    Firestore.instance.collection("users/$uid/payments").add({
      "amount": amount,
      "currency": currency
    });
  }

  static Future<void> _updateWithCustomerId(String customerId) async {
    final String id = "CuDKCDyx9GSzvk9qB02xJsZAKxg2";

    Firestore.instance.document("restaurants/$id").updateData({
      "customer_id": customerId
    });
  }

  static Future<String> _getCustomerId() async {
    final String id = "CuDKCDyx9GSzvk9qB02xJsZAKxg2";

    final DocumentSnapshot snapshot = await Firestore.instance.document("restaurants/$id").get();

    final String customerId = snapshot.data["customer_id"];

    return customerId;
  }

  static Future<void> _payWithNewCard(double amount, String currency) async {
    final bool isSupported = await StripePayment.deviceSupportsNativePay();

    PaymentMethod paymentMethod;

    if (isSupported) {
      paymentMethod = await ClientPaymentService.useNativePay(amount, currency);
    } else {
      paymentMethod = await ClientPaymentService.enterCard();
    }

    final Map<String, dynamic> intent = await ServerPaymentService.getClientSecretForPayment(amount, currency);

    await ClientPaymentService.confirmPayment(paymentMethod.id, intent["client_secret"]);
  }

  static Future<void> _payWithExistingCard(double amount, String currency) async {
    ClientPaymentService.setPublishableKey();

    final String customerId = await _getCustomerId();

    final Map<String, dynamic> intent = await ServerPaymentService.getClientSecretAndPaymentMethod(customerId, amount, currency);

    await ClientPaymentService.confirmPayment(intent["last_payment_error"]["payment_method"]["id"], intent["client_secret"]);
  }

  static Future<void> pay(double amount, String currency, String id) async {
    ClientPaymentService.setPublishableKey();

    if (id != "") {
      await _payWithExistingCard(amount, currency);
    } else {
      await _payWithNewCard(amount, currency);
    }

    await _savePayment(amount, currency);
  }

  static Future<void> setupCard() async {
    ClientPaymentService.setPublishableKey();

    final Map<String, dynamic> intent = await ServerPaymentService.getClientSecretForSetup();

    final PaymentMethod paymentMethod = await ClientPaymentService.enterCard();

    await ClientPaymentService.confirmSetup(paymentMethod, intent["client_secret"]);

    await PaymentService._updateWithCustomerId(intent["customer"]);
  }

}